#!/usr/bin/env python3

# Command line tool for plaintext calender "calculator.py"
# Copyright (C) 2019  Daniel Bauer <daniel.bauer@caserio.de>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import click
import re
import datetime
"""
Groups: start_y start_mon start_d start_h start_min end_y end_mon end_d end_h end_min title category location
         |                Start time             |               End time                     |        Title       |     Category   |      Location      |
"""
regex = "((\d{4})-(\d{2})-(\d{2})/(\d{2}):(\d{2}))\ ?((\d{4})-(\d{2})-(\d{2})/(\d{2}):(\d{2}))? ([a-zA-Z0-9\.\ ,:]*)(\+[a-zA-Z0-9]*)?\ ?(\@[a-zA-Z0-9_\.]*)?"

def new_event(title : str, start : datetime.datetime, end : datetime.datetime = None, category : str = None, location : str = None) -> dict:
	return {"title": title, "start": start, "end": end, "category": category, "location": location}

def time_str(t : datetime) -> str:
	return "{0:04d}-{1:02d}-{2:02d}/{3:02d}:{4:02d}".format(t.year, t.month, t.day, t.hour, t.minute)

def create_line(event : dict) -> str:
	out = time_str(event["start"])
	if event["end"]:
		out += " {}".format(time_str(event["end"]))
	out += " {}".format(event["title"].rstrip())
	if event["category"]:
		out += " +{}".format(event["category"])
	if event["location"]:
		out += " @{}".format(event["location"])
	return out + "\n"

def generate_event(line: str) -> dict:
	data = re.fullmatch(regex, line).groups()
	title = data[12]
	start = datetime.datetime(int(data[1]), int(data[2]), int(data[3]), int(data[4]), int(data[5]))
	#Is end given
	if data[6]:
		end = datetime.datetime(int(data[7]), int(data[8]), int(data[9]), int(data[10]), int(data[11]))
	else:
		end = None
	if data[13]:
		category = data[13][1:]
	else:
		category = None
	if data[14]:
		location = data[14][1:]
	else: 
		location = None
	return new_event(title, start, end, category, location)

def correct_formatted(line : str) -> bool:
	return re.fullmatch(regex, line) != None

CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
@click.group(context_settings=CONTEXT_SETTINGS)
def cli():
	"""
	calendar.py is a simple utility script for managing your plaintext calendar
	The file format documentation is here https://dj-bauer.de/plain-text-kalender.html
	"""
	pass

@cli.command()
@click.argument("filename", metavar="<filename>")
@click.argument("title", metavar="<title>")
@click.argument("start", metavar="<start>")
@click.option("-e", "--end", help="End of the event")
@click.option("-c", "--category", help="Category of the event")
@click.option("-l", "--location", help="Location where the event is")
def add(filename, title, start, end, category, location):
	r = re.match("(\d{4})-(\d{2})-(\d{2})/(\d{2}):(\d{2})", start).groups()
	start = datetime.datetime(int(r[0]), int(r[1]), int(r[2]), int(r[3]), int(r[4]))
	if end:
		r = re.match("(\d{4})-(\d{2})-(\d{2})/(\d{2}):(\d{2})", end).groups()
		end = datetime.datetime(int(r[0]), int(r[1]), int(r[2]), int(r[3]), int(r[4]))
	event = new_event(title, start, end, category, location)
	"""Add an event to the calendar"""
	with open(filename, "a") as f:
		f.write(create_line(event))

@cli.command()
@click.argument("filename", metavar="<filename>")
@click.option("--past-on-top", help="Sorts the past events also and doesn't let them down", is_flag=True)
def sort(filename : str, past_on_top):
	"""Sort calendar file.
	By default it sorts it by date and the past events are whole bottom down.
	"""
	events = []
	with open(filename, "r") as f:
		for line in f.readlines():
			line = line[:len(line)-1]
			if correct_formatted(line):
				events.append(generate_event(line))
	
	if past_on_top:
		sorted_events = sorted(events, key=lambda k : k["start"])
	else:
		old = []
		for event in events:
			if event["start"].date() < datetime.date.today():
				old.append(event)
				events.remove(event)
		sorted_events = sorted(events, key=lambda k : k["start"])
		sorted_old = sorted(old, key=lambda k : k["start"])
	out = ""
	for event in sorted_events:
		out += create_line(event)
	if sorted_old:
		out += "\n"
		for event in sorted_old:
			out += create_line(event)
	with open(filename, "w") as f:
		f.write(out)

@cli.command()
@click.argument("filename", metavar="<filename>")
@click.argument("title", metavar="<title>")
@click.option("--start", help="Start of the event")
def rm(filename, title, start):
	"""Remove event from calendar"""
	print("This feature is not yet supported")

@cli.command()
@click.argument("filename", metavar="<filename>")
@click.option("--today", help="Shows all events for today", is_flag=True)
@click.option("--tomorrow", help="Shows all events for tomorrow", is_flag=True)
@click.option("--this-week", help="Shows all events this week", is_flag=True)
@click.option("-c", "--category", help="Shows all events for specific category")
@click.option("-l", "--location", help="Shows all events for specific location")
@click.option("-a", "-list_all", help="Shows also events in the past", is_flag=True)
def ls(filename, today, tomorrow, this_week, category, location, a):
	"""Show allevents sorted by date"""
	date_today = datetime.date.today()
	date_tomorrow = datetime.date.today() + datetime.timedelta(days=1)
	start_week = date_today - datetime.timedelta(date_today.weekday())
	end_week = start_week + datetime.timedelta(7)
	events = []
	with open(filename, "r") as f:
		for line in f.readlines():
			#Remove newline
			line = line[:len(line)-1]
			if correct_formatted(line):
				event = generate_event(line)
				if category and event["category"] == category:
					events.append(event)
				elif location and event["location"] == location:
					events.append(event)
				elif today and date_today == event["start"].date():
					events.append(event)
				elif tomorrow and date_tomorrow == event["start"].date():
					events.append(event)
				elif this_week and start_week < event["start"].date() < end_week():
					events.append(event)
				elif a:
					events.append(event)
				elif event["start"].date() >= date_today:
					events.append(event)
	for event in events:
		if event["end"]:
			line = "{} {} {}".format(event["start"], event["end"], event["title"])
			if event["category"]:
				line += " +{}".format(event["category"])
			if event["location"]:
					line += " @{}".format(event["location"])
		else:
			line = "{}                     {}".format(event["start"], event["title"])
			if event["category"]:
				line += " +{}".format(event["category"])
			if event["location"]:
				line += " @{}".format(event["location"])
		print(line)

if __name__ == "__main__":
	cli()
